@@ -19,8 +19,6 @@
 #ifdef MANIFOLD_CROSS_SECTION
 #include "manifold/cross_section.h"
 #endif
-#include "../src/tri_dist.h"
-#include "samples.h"
 #include "test.h"
 
 namespace {
@@ -43,13 +41,53 @@ int NumUnique(const std::vector<T>& in) {
  * produces a consistent result.
  */
 TEST(Manifold, GetMeshGL) {
-  Manifold manifold = Manifold::Sphere(1);
+  Manifold manifold = Manifold::Sphere(0.01);
   auto mesh_out = manifold.GetMeshGL();
   Manifold manifold2(mesh_out);
   auto mesh_out2 = manifold2.GetMeshGL();
   Identical(mesh_out, mesh_out2);
 }
 
+TEST(Manifold, MeshDeterminism) {
+  Manifold cube1 = Manifold::Cube(vec3(2.0, 2.0, 2.0), true);
+  Manifold cube2 = Manifold::Cube(vec3(2.0, 2.0, 2.0), true)
+                       .Translate(vec3(-1.1091, 0.88509, 1.3099));
+
+  Manifold result = cube1 - cube2;
+  MeshGL out = result.GetMeshGL();
+
+  uint32_t triVerts[]{0,  2,  7,  0,  10, 1,  0,  6,  10, 0, 1,  2,  1, 3,  2,
+                      1,  5,  3,  1,  11, 5,  0,  7,  6,  6, 7,  8,  6, 8,  13,
+                      10, 12, 11, 1,  10, 11, 11, 13, 5,  6, 12, 10, 6, 13, 12,
+                      13, 9,  5,  13, 8,  9,  11, 12, 13, 4, 2,  3,  4, 3,  5,
+                      4,  7,  2,  4,  5,  8,  4,  8,  7,  9, 8,  5};
+
+  float vertProperties[]{-1,      -1,       -1,     -1,      -1,       1,
+                         -1,      -0.11491, 0.3099, -1,      -0.11491, 1,
+                         -0.1091, -0.11491, 0.3099, -0.1091, -0.11491, 1,
+                         -1,      1,        -1,     -1,      1,        0.3099,
+                         -0.1091, 1,        0.3099, -0.1091, 1,        1,
+                         1,       -1,       -1,     1,       -1,       1,
+                         1,       1,        -1,     1,       1,        1};
+
+  bool flag = true;
+  for (size_t i = 0; i != out.triVerts.size(); i++) {
+    if (out.triVerts[i] != triVerts[i]) {
+      flag = false;
+      break;
+    }
+  }
+
+  for (size_t i = 0; flag && i != out.vertProperties.size(); i++) {
+    if (out.vertProperties[i] != vertProperties[i]) {
+      flag = false;
+      break;
+    }
+  }
+
+  EXPECT_TRUE(flag);
+}
+
 TEST(Manifold, Empty) {
   MeshGL emptyMesh;
   Manifold empty(emptyMesh);
@@ -117,6 +155,55 @@ TEST(Manifold, InvalidInput6) {
   EXPECT_EQ(tet.Status(), Manifold::Error::VertexOutOfBounds);
 }
 
+TEST(Manifold, OppositeFace) {
+  MeshGL gl;
+  gl.vertProperties = {
+      0, 0, 0,  //
+      1, 0, 0,  //
+      0, 1, 0,  //
+      1, 1, 0,  //
+      0, 0, 1,  //
+      1, 0, 1,  //
+      0, 1, 1,  //
+      1, 1, 1,  //
+      //
+      2, 0, 0,  //
+      2, 1, 0,  //
+      2, 0, 1,  //
+      2, 1, 1,  //
+  };
+  gl.triVerts = {
+      0, 1,  4,   //
+      0, 2,  3,   //
+      0, 3,  1,   //
+      0, 4,  2,   //
+      1, 3,  5,   //
+      1, 3,  9,   //
+      1, 5,  3,   //
+      1, 5,  4,   //
+      1, 8,  5,   //
+      1, 9,  8,   //
+      2, 4,  6,   //
+      2, 6,  7,   //
+      2, 7,  3,   //
+      3, 5,  7,   //
+      3, 7,  5,   //
+      3, 7,  11,  //
+      3, 11, 9,   //
+      4, 5,  6,   //
+      5, 7,  6,   //
+      5, 8,  10,  //
+      5, 10, 7,   //
+      7, 10, 11,  //
+      8, 9,  10,  //
+      9, 11, 10,  //
+  };
+  Manifold man(gl);
+  EXPECT_EQ(man.Status(), Manifold::Error::NoError);
+  EXPECT_EQ(man.NumVert(), 12);
+  EXPECT_FLOAT_EQ(man.Volume(), 2);
+}
+
 /**
  * ExpectMeshes performs a decomposition, so this test ensures that compose and
  * decompose are inverse operations.
@@ -233,6 +320,16 @@ TEST(Manifold, Revolve3) {
 }
 #endif
 
+TEST(Manifold, RevolveClip) {
+  Polygons polys = {{{-5, -10}, {5, 0}, {-5, 10}}};
+  Polygons clipped = {{{0, -5}, {5, 0}, {0, 5}}};
+  Manifold first = Manifold::Revolve(polys, 48);
+  Manifold second = Manifold::Revolve(clipped, 48);
+  EXPECT_EQ(first.Genus(), second.Genus());
+  EXPECT_EQ(first.Volume(), second.Volume());
+  EXPECT_EQ(first.SurfaceArea(), second.SurfaceArea());
+}
+
 TEST(Manifold, PartialRevolveOnYAxis) {
   Polygons polys = SquareHole(2.0);
   Polygons offsetPolys = SquareHole(10.0);
@@ -446,11 +543,31 @@ TEST(Manifold, Slice) {
   EXPECT_EQ(bottom.Area(), 1);
   EXPECT_EQ(top.Area(), 0);
 }
+
+TEST(Manifold, SliceEmptyObject) {
+  Manifold empty;
+  EXPECT_TRUE(empty.IsEmpty());
+  CrossSection bottom = empty.Slice();
+}
+
+TEST(Manifold, Simplify) {
+  Polygons polyCircle =
+      CrossSection::Circle(1, 20).Translate({10, 0}).ToPolygons();
+  Manifold torus = Manifold::Revolve(polyCircle, 100);
+  Manifold simplified = torus.Simplify(0.4);
+  EXPECT_NEAR(torus.Volume(), simplified.Volume(), 20);
+  EXPECT_NEAR(torus.SurfaceArea(), simplified.SurfaceArea(), 10);
+
+#ifdef MANIFOLD_EXPORT
+  if (options.exportModels) ExportMesh("torus.glb", simplified.GetMeshGL(), {});
+#endif
+}
 #endif
 
 TEST(Manifold, MeshRelation) {
-  MeshGL gyroidMeshGL = WithPositionColors(Gyroid()).AsOriginal().GetMeshGL();
-  Manifold gyroid(gyroidMeshGL);
+  Manifold gyroid = WithPositionColors(Gyroid());
+  MeshGL gyroidMeshGL = gyroid.GetMeshGL();
+  gyroid = gyroid.Simplify();
 
 #ifdef MANIFOLD_EXPORT
   ExportOptions opt;
@@ -600,7 +717,8 @@ TEST(Manifold, MergeEmpty) {
   EXPECT_TRUE(shape.Merge());
   Manifold man(shape);
   EXPECT_EQ(man.Status(), Manifold::Error::NoError);
-  EXPECT_TRUE(man.IsEmpty());
+  EXPECT_EQ(man.NumTri(), 4);
+  EXPECT_TRUE(man.Simplify().IsEmpty());
 }
 
 TEST(Manifold, PinchedVert) {
@@ -638,7 +756,7 @@ TEST(Manifold, FaceIDRoundTrip) {
   const Manifold cube = Manifold::Cube();
   ASSERT_GE(cube.OriginalID(), 0);
   MeshGL inGL = cube.GetMeshGL();
-  ASSERT_EQ(NumUnique(inGL.faceID), 12);
+  ASSERT_EQ(NumUnique(inGL.faceID), 6);
   inGL.faceID = {3, 3, 3, 3, 3, 3, 5, 5, 5, 5, 5, 5};
 
   const Manifold cube2(inGL);
@@ -990,4 +1108,4 @@ TEST(Manifold, MergeRefine) {
   manifold = manifold.RefineToLength(1.0);
   EXPECT_NEAR(manifold.Volume(), 31.21, 0.01);
 }
-#endif
\ No newline at end of file
+#endif
